#ifndef DOUBDP_H
#define DOUBDP_H

#include "alias.h"
#include "iotools.h"
#include "strretalg.h"
struct matrix;
struct vector;
struct atsel;

/**
   class doubdp defines plasticity material model for concrete
   including double Drucker-Prager criterium
   
   Josef Fiedler, 08/2014
*/

class doubdp
{
 public:
  doubdp (void);
  ~doubdp (void);
  void read (XFILE *in);

  void   params (long ipp, vector &q);
  double yieldfunction  (matrix &sig, vector &q);
  void   deryieldfsigma (matrix &sig, matrix &dfds, vector &q);
  void   matstiff       (matrix &d, long ipp,long ido);
  void   tangentstiff   (matrix &d,matrix &td,long ipp,long ido);
  void   nlstresses     (long ipp, long im, long ido);
  void   updateval      (long ipp, long im, long ido);
  double plasmodscalar  (matrix &sig, vector &q);
  void   updateq        (long ipp, double dgamma, vector &q);

  /// strength in single compression
  double fc;
  /// strength in single tension
  double ft;
  /// strength in double compression
  double fb;
  /// alpha parameter for tensile DP
  double alphat;
  /// alpha parameter for compressive DP
  double alphac;
  /// tau parameter for tensile DP
  double taut;
  /// tau parameter for compressive DP
  double tauc;
  /// I1 value of intersection point
  double icr;
  /// J2 value of intersection point
  double jcr;
  /// I1 value of top point of cone
  double ivrch;
  /// J2 value of top point of cone
  double jvrch;
  /// switching parameter
  long v;
  /// concrete energy of cracking
  double gtf;
  /// gammma for ultimate cracking
  double gamult;

  ///  stress return algorithm
  strretalg sra;
  
  

};

#endif
