#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "layplate.h"
#include "global.h"
#include "vecttens.h"
#include "intpoints.h"
#include "matrix.h"
#include "vector.h"
#include "tensor.h"
#include "elastisomat.h"
#include "strretalg.h"
#include "alias.h"

/**
  This constructor inializes attributes to zero values.
*/
layplate::layplate (void)
{
  
}


/**
  The destructor
*/
layplate::~layplate (void)
{

}


/**
  This function reads material parameters from the opened text file given
  by the parameter in.

  @param in - pointer to the opened text file

  10/2014   J. Fiedler
*/
void layplate::read (XFILE *in)
{
  long i,j;
//  long eid = Mm->elip[ipp];
  
  xfscanf (in,"%k%ld", "num_lay", &nl);
/*  if (Mc->give_num_lay(eid)!=nl){
    print_err("wrong number of layers defined", __FILE__, __LINE__, __func__);
  }
*/    
  tm  = new mattype*[nl];
  idm = new long*[nl];
  nm  = new long[nl];
  
  for (i=0;i<nl;i++){                       // cyklus pres vrstvy
    xfscanf (in,"%ld", &nm[i]);             // nacteni poctu materialu na vrstve
    tm[i]  = new mattype[nm[i]];            // alokovani tm
    idm[i] = new long[nm[i]];               // alokovani idm

    for (j=0;j<nm[i];j++){
      xfscanf(in, "%k%m%k%ld", "mattype", &mattype_kwdset, (int*)(tm[i]+j), "num_inst", idm[i]+j);
      idm[i][j]--;
    }
  }
    
}



/**
  This function computes material stiffnes %matrix.
*/
void layplate::matstiff (matrix &d, long ipp,long ido)
{
  long i,j,jj;
  double *layth;
  double *layz;
  double t,th = 0.0;
  matrix e(4,4);
  long eid = Mm->elip[ipp];
  double g, gc;
  double *k;

/*  if (Mp->nlman == NULL)
    Mm->elmatstiff (d,ipp);
  else
  {
    if (Mp->nlman->stmat==initial_stiff)
    {
    //  initial elastic matrix
     Mm->elmatstiff (d,ipp);
    }
    if (Mp->nlman->stmat==tangent_stiff)
    {
    //  tangent stiffness matrix
//    matrix ad(d.m,d.n);
    Mm->elmatstiff (d,ipp);
//    tangentstiff (ad,d,ipp,ido);
    }
  }

  layth = Mc->give_layer_thicke(eid);
  layz  = Mc->give_layer_zcoord(eid);  
  for (i=0;i<nl;i++){
    th += layth[i];
  }
  t = th*th*th;

  cmulm(t,d);

*/
layth = Mc->give_layer_thicke(eid);
layz  = Mc->give_layer_zcoord(eid);

 gc = 0.0;  
fillm(0,e);         /// proc jsou funkce pro matrix long???
fillm(0,d);         /// proc jsou funkce pro matrix long???
backup(ipp,k,0);      /// vyresit krivosti ... motaj se mi tam
for (i=0;i<nl;i++){
  Mm->ip[ipp].tm  = tm [i];     // mattype aktualni vrstvy
  Mm->ip[ipp].idm = idm[i];     // index sady mat. vl. aktualni sady
  Mm->ip[ipp].nm  = nm [i];
  
  Mm->elmatstiff (e,ipp);
  g = layth[i]*layz[i]*layz[i];
  gc += g;
  cmulm(g,e);
  for (j=0;j<d.m;j++){
    for (jj=0;jj<d.n;jj++){
      d[j][jj] += e[j][jj];
    }
  }
}
Mm->elmatstiff (e,ipp);
cmulm(gc, e);

restore_values(ipp,k,0);

}


/**
  This function computes stresses at given integration point ipp,
  depending on the reached strains. The stress and the other attribute of
  given integration point is actualized.

  @param ipp - integration point number in the mechmat ip array.
  @param im  - index of material type for given ip
  @param ido - index of internal variables for given material in the ipp other array

  @return The function updates the stress array of the given integration point.

  10/2014 J. Fiedler
*/
void layplate::nlstresses (long ipp, long im, long ido)
{
  long i,j,nc = Mm->ip[ipp].ncompstr;
  long eid = Mm->elip[ipp];
  double *k;          // vektor krivosti
  double *layth;      // ukazatel na pole tloustek
  double *layz;       // ukazatel na pole z-tovych souradnic
  vector intfor(nc);  // vnitrni sily - momenty
  
  //  ulozeni potrebnych ukazatelu
  layth = Mc->give_layer_thicke(eid);
  layz  = Mc->give_layer_zcoord(eid);  

  backup (ipp,k,1);                   // zaloha informaci na integracnim bode
  ido = 0;

  // hlavni cyklus pres vrstvy
  for (i=0;i<nl;i++){
    Mm->ip[ipp].tm  = tm [i];     // mattype aktualni vrstvy
    Mm->ip[ipp].idm = idm[i];     // index sady mat. vl. aktualni sady
    Mm->ip[ipp].nm  = nm [i];

    for (j=0;j<nc;j++){           // cyklus pres slozky deformaci -> vypocet deformace pro prislusnou vrstvu
      Mm->ip[ipp].strain[j] = k[j]*layz[i];
    }
    
    Mm->computenlstresses(ipp,0,ido);
    
    for (j=0;j<nc;j++){
      intfor[j] += Mm->ip[ipp].stress[j]*layth[i]*layz[i];  // nascitavani momentu
    }

    ido += Mm->givencompeqother(ipp,0);
  }
  Mm->storestress(0, ipp, intfor);
  restore_values (ipp,k,1);         // vraceni na puvodni hodnoty
}


/**
  This function evaluates the size of eqother array.

  @param ipp - integration point number in the mechmat ip array.

  @return The function returns the size of eqother array depending of number of layers
          and their material type.

  10/2014 J. Fiedler
*/
long layplate::compeqother(long ipp)  // -> zavolat tuhle funkci v mechmatu ve funkci givencomeqother
{
  long sizeeqother=0;
  long i;
  mattype *btm = Mm->ip[ipp].tm;
  Mm->ip[ipp].ncompstr = 4;

  // cyklus pres vrstvy
  for (i=0;i<nl;i++){
    Mm->ip[ipp].tm  = tm [i];
    sizeeqother += Mm->givencompeqother(ipp,0);
  }

  Mm->ip[ipp].tm  = btm;  // vraci mattype na puvodni layplate
  Mm->ip[ipp].ncompstr = 3;
  
  return sizeeqother;
}


/**
  This function evaluates the size of other array.

  @param ipp - integration point number in the mechmat ip array.

  @return The function returns the size of other array depending of number of layers
          and their material type.

  10/2014 J. Fiedler
*/
long layplate::compother(long ipp)  // -> zavolat tuhle funkci v mechmatu ve funkci givencompother
{
  long sizeother=0;
  long i;
  mattype *btm = Mm->ip[ipp].tm;
  Mm->ip[ipp].ncompstr = 4;

  // cyklus pres vrstvy
  for (i=0;i<nl;i++){
    Mm->ip[ipp].tm  = tm[i];
    sizeother += Mm->givencompother(ipp,0);
  }

  Mm->ip[ipp].tm  = btm;  // vraci mattype na puvodni layplate
  Mm->ip[ipp].ncompstr = 3;
  
  return sizeother;
}


/**
  Function backs up some informations stored on integration point and prepares arrays for computation.

  @param ipp - integration point number in the mechmat ip array.

  10/2014 J. Fiedler
*/
void layplate::backup(long ipp, double *&k, long j)
{
  bcup.tm   = Mm->ip[ipp].tm;
  bcup.idm  = Mm->ip[ipp].idm;
  bcup.nm   = Mm->ip[ipp].nm;
  bcup.ssst = Mm->ip[ipp].ssst;
  bcup.ncompstr = Mm->ip[ipp].ncompstr;
  Mm->ip[ipp].ssst = planestress;
  Mm->ip[ipp].ncompstr = 4;

  if (j==1)
  {
    bcup.stress = Mm->ip[ipp].stress;
    k = Mm->ip[ipp].strain;
    Mm->ip[ipp].alloc_strains(1);   // znovualokovani podle noveho stress-strain state
    Mm->ip[ipp].alloc_stresses(1);
  }
}

/**
  Function restores original informations stored on integration point.

  @param ipp - integration point number in the mechmat ip array.

  10/2014 J. Fiedler
*/
void layplate::restore_values(long ipp, double *k, long j)
{
  long i;
  
  Mm->ip[ipp].tm       = bcup.tm;
  Mm->ip[ipp].idm      = bcup.idm;
  Mm->ip[ipp].nm       = bcup.nm;
  Mm->ip[ipp].ssst     = bcup.ssst;
  Mm->ip[ipp].ncompstr = bcup.ncompstr;

  if (j==1)
  {
    delete [] Mm->ip[ipp].strain;
    Mm->ip[ipp].strain = k;
    for (i=0; i<3; i++)
      bcup.stress[i] = Mm->ip[ipp].stress[i];
    delete [] Mm->ip[ipp].stress;
    Mm->ip[ipp].stress = bcup.stress;
  }
}


/**
  This function updates values in the other array reached in the previous equlibrium state to
  values reached in the new actual equilibrium state.
*/
void layplate::updateval (long ipp, long im, long ido)
{
  long i,n = Mm->givencompeqother(ipp,im);

  for (i=0;i<n;i++)
    Mm->ip[ipp].eqother[ido+i]=Mm->ip[ipp].other[ido+i];
}

